﻿
namespace BeeDevelopment.Raytracer {
	
	/// <summary>Describes what happened during a collision.</summary>
	public struct Collision {

		#region Types

		/// <summary>Defines the direction of the collision.</summary>
		public enum CollisionDirection {
			/// <summary>The ray is entering the solid at the point of the collision.</summary>
			EnteringSolid,
			/// <summary>The ray is leaving the solid at the point of the collision.</summary>
			LeavingSolid,
		}

		#endregion

		#region Properties

		private CollisionDirection direction;
		/// <summary>Gets or sets the direction of the collision.</summary>
		public CollisionDirection Direction { get { return this.direction; } set { this.direction = value; } }

		private Vector3 position;
		/// <summary>The position that was hit.</summary>
		public Vector3 Position { get { return this.position; } set { this.position = value; } }

		private double distance;
		/// <summary>The distance along the path that the collision takes place.</summary>
		/// <remarks>0 corresponds to the start of the path, 1 corresponds to the end.</remarks>
		public double Distance { get { return this.distance; } set { this.distance = value; } }

		private Vector3 surfaceNormal;
		/// <summary>Gets or sets the normal of the surface that the ray collides with.</summary>
		public Vector3 SurfaceNormal { get { return this.surfaceNormal; } set { this.surfaceNormal = value; } }


		public Vector3 CollisionNormal {
			get {
				return this.direction == CollisionDirection.EnteringSolid ? this.surfaceNormal : -this.surfaceNormal;
			}
		}

		#endregion

		/// <summary>Create an instance of the collision structure.</summary>
		public Collision(CollisionDirection direction, Vector3 position, double distance, Vector3 surfaceNormal) {
			this.direction = direction;
			this.position = position;
			this.distance = distance;
			this.surfaceNormal = surfaceNormal;
		}

	}
}
