﻿
namespace BeeDevelopment.Raytracer.Lights {

	/// <summary>
	/// Represents a spotlight.
	/// </summary>
	/// <remarks>Points within the inner angle are lit fully; points outside the outer angle are not lit at all. Points between the two are blended accordingly.</remarks>
	public class Spotlight : PointLight {

		/// <summary>
		/// Gets or sets the direction that the light is pointing.
		/// </summary>
		public Vector3 Direction { get; set; }

		/// <summary>
		/// Gets or sets the angle of the spotlight's inner beam.
		/// </summary>
		public double InnerAngleLimit { get; set; }

		/// <summary>
		/// Gets or sets the angle of the spotlight's outer beam.
		/// </summary>
		public double OuterAngleLimit { get; set; }


		/// <summary>
		/// Creates an instance of a <see cref="Spotlight"/>.
		/// </summary>
		/// <param name="origin">The origin of the spotlight.</param>
		/// <param name="direction">The direction that the spotlight points in.</param>
		/// <param name="angle">The angle of the spotlight's beam.</param>
		public Spotlight(Vector3 origin, Vector3 direction, double angle)
			: base(origin) {
			this.Direction = direction;
			this.InnerAngleLimit = angle;
			this.OuterAngleLimit = angle;
		}

		/// <summary>
		/// Creates an instance of a <see cref="Spotlight"/>.
		/// </summary>
		/// <param name="origin">The origin of the spotlight.</param>
		/// <param name="direction">The direction that the spotlight points in.</param>
		/// <param name="innerAngle">The inner angle of the spotlight's beam.</param>
		/// <param name="outerAngle">The outer angle of the spotlight's beam.</param>
		public Spotlight(Vector3 origin, Vector3 direction, double innerAngle, double outerAngle)
			: this(origin, direction, 0) {
			this.InnerAngleLimit = innerAngle;
			this.OuterAngleLimit = outerAngle;
		}

	}
}
