﻿using System.Drawing;

namespace BeeDevelopment.Raytracer {
	public class Material {

		/// <summary>
		/// Gets or sets the colour of the material.
		/// </summary>
		public Colour Colour { get; set; }

		/// <summary>
		/// Gets or sets the reflectivity of the surface.
		/// </summary>
		public double Reflectivity { get; set; }

		/// <summary>
		/// Gets or sets the refractive index of the surface.
		/// </summary>
		public double RefractiveIndex { get; set; }

		/// <summary>
		/// Gets or sets the texture of the surface.
		/// </summary>
		public ITexture Texture { get; set; }

		/// <summary>
		/// Gets true if the material prevents a ray from moving any further.
		/// </summary>
		public bool IsRayTerminator {
			get {
				return Reflectivity <= 0.0d && Colour.A >= 1.0d;
			}
		}

		public Material() {
			this.Colour = Colour.White;
			this.RefractiveIndex = 1.0d;
		}

	}
}
