﻿namespace BeeDevelopment.Raytracer {

	/// <summary>
	/// Defines a simple procedural texture.
	/// </summary>
	public class ProceduralTexture : ITexture {

		/// <summary>
		/// A delegate used to generate textures.
		/// </summary>
		/// <param name="coordinate">The texture coordinate to retrieve a colour from.</param>
		/// <returns>The colour generated at the specified texture coordinate.</returns>
		public delegate Colour ProceduralTextureGenerator(Vector2 coordinate);

		/// <summary>
		/// Gets or sets the <see cref="ProceduralTextureGenerator"/> that generates the texture.
		/// </summary>
		public ProceduralTextureGenerator Generator { get; set; }

		/// <summary>
		/// Creates an instance of a <see cref="ProceduralTexture"/>.
		/// </summary>
		/// <param name="generator">The <see cref="ProceduralTextureGenerator"/> that generates the texture.</param>
		public ProceduralTexture(ProceduralTextureGenerator generator) {
			this.Generator = generator;
		}

		/// <summary>
		/// Gets a colour from the texture.
		/// </summary>
		/// <param name="coordinate">The texture coordinate to retrieve the colour for.</param>
		/// <returns>The colour of the texture at the specified point.</returns>
		public Colour GetColour(Vector2 coordinate) {
			return Generator(coordinate);
		}
	}
}
