﻿using System;
using System.Collections.Generic;
using System.Text;

namespace BeeDevelopment.Raytracer.Surfaces.Csg {
	/// <summary>
	/// Represents the subtraction of two surfaces.
	/// </summary>
	/// <remarks>Surface B is subtracted from surface A.</remarks>
	public class Subtraction : IRayCollidable, ITexturable {

		/// <summary>
		/// Gets the operand surface A.
		/// </summary>
		public IRayCollidable A { get; set; }

		/// <summary>
		/// Gets the operand surface B.
		/// </summary>
		public IRayCollidable B { get; set; }

		public Subtraction(IRayCollidable a, IRayCollidable b) {
			this.A = a;
			this.B = b;
		}

		#region IRayCollidable Members

		public Collision[] GetCollision(Ray ray) {
			

			// Get all the collisions that strike both surfaces.
			var ACollisions = new List<Collision>(A.GetCollision(ray) ?? new Collision[0]);
			var BCollisions = new List<Collision>(B.GetCollision(ray) ?? new Collision[0]);
			
			// We don't want collisions that happened in A that are also inside B.
			ACollisions.RemoveAll(c => B.PointIsInside(c.Position));
			// We also don't want collisions that happened in B that are outside A.
			BCollisions.RemoveAll(c => !A.PointIsInside(c.Position));

			// Return all collisions.
			var AllCollisions = new List<Collision>();
			AllCollisions.AddRange(ACollisions);
			AllCollisions.AddRange(BCollisions);
			return AllCollisions.ToArray();
		}

		public bool PointIsInside(Vector3 point) {
			return A.PointIsInside(point) && !B.PointIsInside(point);
		}

		public Vector3 GetClosestPoint(Vector3 point) {
			return Vector3.Zero;
		}

		#endregion

		#region ITexturable Members

		public Vector2 GetTextureCoordinate(Vector3 surfacePoint) {
			// Which surface have we collided with?
			if (A.PointIsInside(surfacePoint) && B.PointIsInside(surfacePoint)) {
				if (B is ITexturable) {
					return (B as ITexturable).GetTextureCoordinate(surfacePoint);
				} else {
					return Vector2.Zero;
				}
			} else {
				if (A is ITexturable) {
					return (A as ITexturable).GetTextureCoordinate(surfacePoint);
				} else {
					return Vector2.Zero;
				}
			}
		}

		#endregion
	}
}
