﻿using System;
using System.Collections.Generic;
using System.Text;

namespace BeeDevelopment.Raytracer.Surfaces {
	public struct Sphere : IRayCollidable, ITexturable {

		private Vector3 origin;
		/// <summary>Gets or sets the origin of the sphere.</summary>
		public Vector3 Origin { get { return this.origin; } set { this.origin = value; } }

		private double radius;
		/// <summary>Gets or sets the radius of the sphere.</summary>
		public double Radius { get { return this.radius; } set { this.radius = value; } }

		public Sphere(Vector3 origin, double radius) {
			this.origin = origin;
			this.radius = radius;
		}

		public Collision[] GetCollision(Ray ray) {

			double b = Vector3.Dot(ray.Direction, ray.Origin - this.origin);
			double c = Vector3.DistanceSquared(ray.Origin, this.origin) - (this.radius * this.radius);
			double d = b * b - c;

			if (d < 0.0f) {
				return null;
			} else {

				var Collisions = new List<Collision>(2);

				for (int m = -1; m <= +1; m += 2) {
					double Distance = -b + (m * Math.Sqrt(d));
					if (Distance >= 0) {
						var CollisionPoint = ray.Origin + (Distance * ray.Direction);
						Collisions.Add(new Collision(m == -1 ? Collision.CollisionDirection.EnteringSolid : Collision.CollisionDirection.LeavingSolid, CollisionPoint, Distance, Vector3.Normalise(CollisionPoint - this.origin)));
					}
				}
				return Collisions.ToArray();				
			}			
		}

		public bool PointIsInside(Vector3 point) {
			return (point - this.Origin).MagnitudeSquared < (this.radius * this.radius);
		}

		public Vector3 GetClosestPoint(Vector3 point) {
			return Vector3.Normalise(point - this.origin) * this.radius + this.origin;
		}

		public Vector2 GetTextureCoordinate(Vector3 surfacePoint) {

			var SurfaceOffset = surfacePoint - this.origin;

			Vector2 Angle = new Vector2(
				-Math.Atan2(SurfaceOffset.X, SurfaceOffset.Z),
				-Math.Acos(SurfaceOffset.Y / this.radius)
			);

			return new Vector2(Angle.X / (Math.PI * 2.0d), Angle.Y / Math.PI);
		}
	}
}
