﻿
namespace BeeDevelopment.Raytracer {
	public class WorldObject {

		/// <summary>
		/// Gets or sets the surface associated with the object.
		/// </summary>
		public IRayCollidable Surface { get; set; }

		/// <summary>
		/// Gets or sets the material of the object.
		/// </summary>
		public Material Material { get; set; }

		/// <summary>
		/// Gets the colour at a particular point on the surface of the object.
		/// </summary>
		/// <param name="point">The point to get the colour of.</param>
		/// <returns>Colour at that point.</returns>
		public Colour GetColour(Vector3 point) {
			if (this.Material == null) return Colour.White;
			if (this.Material.Texture != null && this.Surface != null && this.Surface is ITexturable) {
				return this.Material.Colour * this.Material.Texture.GetColour((this.Surface as ITexturable).GetTextureCoordinate(point));
			} else {
				return this.Material.Colour;
			}
		}

		/// <summary>
		/// Creates an instance of a <see cref="WorldObject"/>.
		/// </summary>
		public WorldObject() { }

		/// <summary>
		/// Creates an instance of a <see cref="WorldObject"/> from a surface and material.
		/// </summary>
		/// <param name="surface">The surface associated with the object.</param>
		/// <param name="material">The material of the object.</param>
		public WorldObject(IRayCollidable surface, Material material) {
			this.Surface = surface;
			this.Material = material;
		}

	}
}
